------------------------------------------------+
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use std.standard.all;
------------------------------------------------+
entity  LCD_MT12864J_drv is

Port (  DATA:       in   unsigned(7 downto 0):="10101010";
		Point_Bit:  in   std_logic;
		X:          in   unsigned(7 downto 0);
		Y:          in   unsigned(7 downto 0);
		Clk1MHz:    in   std_logic;
		Point_SET:  in   std_logic;
		Clear_SET:  in   std_logic;
		Byte_SET:   in   std_logic;
		--------------------------------
		LCD_CE:     out  unsigned(1 downto 0);
		LCD_LED:    out  std_logic :='0';
		LCD_D:      inout unsigned(7 downto 0);
		LCD_RESET:  out  std_logic :='0';
		LCD_RW:     out  std_logic :='0';
		LCD_A:      out  std_logic :='0';
		LCD_E:      out  std_logic :='0';
		STATUS:     out  std_logic :='1');

end LCD_MT12864J_drv;
------------------------------------------------+
architecture G of LCD_MT12864J_drv is
--=============================================================
	constant STOP:                      integer := 0;
	constant START:                     integer := 1;
---------------------------------------------------
	constant POINT:                     integer := 0;
	constant BYTE:                      integer := 1;
---------------------------------------------------
    signal L,R,CD,LCDRW:                std_logic :='0';
	signal DataW,DataR,MaskBit:         unsigned (7 downto 0):= x"00";
---------------------------------------------------
    signal PointBit:                    std_logic :='0';
    signal STATUSz:                     std_logic :='1';
	signal PointData,PointY:            unsigned (7 downto 0):= x"00";
	signal PointX,PointXmax,PointXtmp:  unsigned (7 downto 0):= x"00";
	signal ByteData, ByteDataTmp:       unsigned (7 downto 0):= x"00";
---------------------------------------------------
    signal ClearSETz,ClearSET:         std_logic :='0';
    signal PointSETz,PointSET:         std_logic :='0';
    signal CharSETz,CharSET:           std_logic :='0';
    signal ByteSETz,ByteSET:           std_logic :='0';
---------------------------------------------------
    signal CNT:                         integer range 0 to 1;
    signal Page:                        unsigned (7 downto 0):= x"00";
    signal Bite:                        integer range 0 to 7;
---------------------------------------------------
	signal InitLCD:                     integer range 0 to 7  :=START;
	signal CLEAR:                       integer range 0 to 3  :=STOP;
	signal STATE:                       integer range 0 to 15 :=STOP;
	signal CharS:                       integer range 0 to 3  :=STOP;
	signal WriteCode:                   integer range 0 to 3  :=STOP;
	signal WriteData:                   integer range 0 to 3  :=STOP;
	signal ReadData:                    integer range 0 to 7  :=STOP;
	signal WaitReady:                   integer range 0 to 3  :=STOP;
	signal Method:                      integer range 0 to 31 :=STOP;
--=============================================================
BEGIN

STATUS <= STATUSz;

-----------------
process (Clk1MHz)
begin
if Clk1MHz'event and Clk1MHz='1' then
--=====================================================================
ClearSETz <= Clear_SET;
ClearSET  <= (ClearSETz and not Clear_SET);
--=====================================================================
--PointSETz <= Point_SET;
--PointSET  <= (PointSETz and not Point_SET);
--=====================================================================
ByteSETz <= Byte_SET;
ByteSET  <= (ByteSETz and not Byte_SET);
--=====================================================================
   if ClearSET = '1' then CLEAR <= START;
                          ByteDataTmp <= x"00";
                          PointX<= x"00"; 
                          PointY<= x"00";
                          Method <= BYTE;
elsif Point_SET = '1' then STATE <= START;
                           PointX <= "0"&X(6 downto 0);    
                           PointY <= "00"&Y(5 downto 0);
                           PointBit <= Point_Bit;
                           Method <= POINT;
                           STATUSz <= '0';
elsif ByteSET = '1'  then STATE <= START;
                          ByteData <= x"FF";
                          ByteDataTmp <= DATA;
                          PointX <= "0"&X(6 downto 0);    
                          PointY <= "00"&Y(5 downto 0);
                          PointBit <= '0';
                          Method <= BYTE;
                     end if;
--=====================================================================
   if CLEAR = 1 then STATE <= START;
                     PointBit <= not PointBit;--'0';
                     ByteData <= x"FF";
                     CLEAR <= 2;
                     if PointX >= x"80" then PointX <= x"00";
                                             if PointY >= x"40" then PointY <= x"00"; 
                                                                     CLEAR <= STOP;
                                                                else PointY <= PointY + x"08";
                                                                end if;
                                        else PointX <= PointX + x"01";
                                        end if;
                         ----------------------------------------------
elsif CLEAR = 2 then if STATE = STOP then CLEAR <= START; 
                                     end if;
                end if;
--=====================================================================
   if STATE = STOP  then CNT <= 0;
elsif STATE = START then if InitLCD = STOP then STATE <= 2; end if;
                         if PointX < x"40" then L <= '1'; R <= '0';
                                                PointXtmp <= PointX;
                                           else L <= '0'; R <= '1';
                                                PointXtmp <= PointX - x"40";
                                           end if;
                         ----------------------------------------------
                      if Method = 0 then Bite <= to_integer(PointY and x"07");
                   elsif Method = 1 then Bite <= to_integer(PointY - (PointY(7 downto 3) & "000"));
                                    end if;
                      Page <= SHIFT_RIGHT (PointY,3);
                      MaskBit <= x"00";
                      CNT <= 0;
                         ----------------------------------------------
elsif STATE = 2  then WriteCode <= START;
                      DataW <= x"B8" or Page;
                      STATE <= 3;                                   
                         ----------------------------------------------
elsif STATE = 3  then if WriteCode = STOP then STATE <= 4;
                                               WriteCode <= START;
                                               DataW <= x"40" or PointXtmp;
                                          end if;
                         ----------------------------------------------
elsif STATE = 4  then if WriteCode = STOP then ReadData <= START;
                                               STATE <= 5; 
                                          end if;
                         ----------------------------------------------
elsif STATE = 5  then if ReadData = STOP  then PointData <= DataR;
                                               if Method = 0 then STATE <= 11; 
                                            elsif Method = 1 then STATE <= 6; 
                                                             end if;
                                          end if;
                         --===BYTE=====================================
elsif STATE = 6  then WriteCode <= START;
                      DataW <= x"40" or PointXtmp;
                      STATE <= 7;
                         ----------------------------------------------
elsif STATE = 7  then if WriteCode = STOP then WriteData <= START;
                                               STATE <= 8;
                                          end if;
                         ----------------------------------------------
elsif STATE = 8  then if PointBit = '1' then if CNT = 0 then DataW <= PointData or (SHIFT_LEFT  (ByteData, Bite));
                                                        else DataW <= PointData or (SHIFT_RIGHT (ByteData, Bite));
                                                        end if;       
                                        else if CNT = 0 then DataW <= PointData and not (SHIFT_LEFT  (ByteData, Bite));
                                                        else DataW <= PointData and not (SHIFT_RIGHT (ByteData, Bite));
                                                        end if;
                                        end if;
                      STATE <= 9; 
                         ----------------------------------------------
elsif STATE = 9  then if WriteData = STOP then STATE <= 10; end if;
                         ----------------------------------------------
elsif STATE = 10 then if Bite > 0 and CNT  = 0 then Page <= Page + x"01";
                                                    Bite <= 8 - Bite;
                                                    CNT <= 1;
                                                    STATE <= 2;
                                               else STATE <= 15;
                                                    CNT <= 0;
                                               end if;
                         ----------------------------------------------
elsif STATE = 15 then if PointBit = '0' then CNT <= 0;
                                             STATE <= 1;
                                             PointBit <= '1';
                                             ByteData <= ByteDataTmp;
                                        else STATE <= STOP; STATUSz <= '1';
                                             CNT <= 0;
                                        end if;
                         --===POINT====================================
elsif STATE = 11 then MaskBit(Bite) <= '1';
                      STATE <= 12;
                         ----------------------------------------------
elsif STATE = 12 then if PointBit = '1' then PointData <= PointData or MaskBit;
                                        else PointData <= PointData and not MaskBit;
                                        end if;
                      WriteCode <= START;
                      DataW <= x"40" or PointXtmp;
                      STATE <= 13;
                         ----------------------------------------------
elsif STATE = 13 then if WriteCode = STOP then STATE <= 14;
                                               WriteData <= START;
                                               DataW <= PointData;
                                          end if;
                         ----------------------------------------------
elsif STATE = 14 then if WriteData = STOP then STATE <= STOP; STATUSz <= '1';
                                          end if;
                         ----------------------------------------------
                 end if;
--=====================================================================
   if InitLCD = 1  then LCD_E <= '0';
                        LCD_RESET <= '0';
                        InitLCD <= 2;
                         ----------------------------------------------
elsif InitLCD = 2  then LCD_RESET <= '1';
                        InitLCD <= 3;
                         ----------------------------------------------
elsif InitLCD = 3  then WriteCode <= START; 
                        DataW <= x"C0";
                        L <= '1'; R <= '0';
                        InitLCD <= 4;
                         ----------------------------------------------
elsif InitLCD = 4  then if WriteCode = STOP then InitLCD <= 5;
                                                 WriteCode <= START;
                                                 DataW <= x"C0";
                                                 L <= '0'; R <= '1';
                                            end if;
                         ----------------------------------------------
elsif InitLCD = 5  then if WriteCode = STOP then InitLCD <= 6;
                                                 WriteCode <= START; 
                                                 DataW <= x"3F";
                                                 L <= '1'; R <= '0';
                                            end if;
                         ----------------------------------------------
elsif InitLCD = 6  then if WriteCode = STOP then InitLCD <= 7;
                                                 WriteCode <= START; 
                                                 DataW <= x"3F";
                                                 L <= '0'; R <= '1';
                                            end if;
                         ----------------------------------------------
elsif InitLCD = 7  then if WriteCode = STOP then InitLCD <= STOP; 
                                                 LCD_LED <= '1';
                                            end if;
                   end if;
--=====================================================================
   if WriteCode  = 1 then CD <= '0';
                          WaitReady <= START; 
                          WriteCode <= 2;
                         ----------------------------------------------
elsif WriteCode  = 2 then if WaitReady = STOP then WriteCode <= 3;
                                                   LCDRW <= '0';
                                                   LCD_A <= CD;
                                                   LCD_CE(0)<= L;
                                                   LCD_CE (1)<= R;
                                                   LCD_E <= '1';
                                              end if;
                         ----------------------------------------------
elsif WriteCode  = 3 then LCD_E <= '0';
                          WriteCode <= STOP;
                         ----------------------------------------------
                     end if;
--=====================================================================
   if WriteData  = 1 then CD <= '1';
                          WaitReady <= START; 
                          WriteData <= 2;
                         ----------------------------------------------
elsif WriteData  = 2 then if WaitReady = STOP then WriteData <= 3;
                                                   LCDRW <= '0';
                                                   LCD_A <= CD;
                                                   LCD_CE(0)<= L;
                                                   LCD_CE (1)<= R;
                                                   LCD_E <= '1';
                                              end if;
                         ----------------------------------------------
elsif WriteData  = 3 then LCD_E <= '0';
                          WriteData <= STOP;
                         ----------------------------------------------
                     end if;
--=====================================================================
   if ReadData  = 1 then WaitReady <= START;  
                         CD <= '1'; 
                         ReadData <= 2;
                         ----------------------------------------------
elsif ReadData  = 2 then if WaitReady = STOP then ReadData <= 3; 
                                                  LCDRW <= '1';
                                                  LCD_A <= CD;
                                                  LCD_CE(0)<= L;
                                                  LCD_CE (1)<= R;
                                                  LCD_E <= '1';
                                             end if;
                         ----------------------------------------------
elsif ReadData  = 3 then WaitReady <= START;  
                         LCD_E <= '0'; 
                         ReadData <= 4;                   
                         ----------------------------------------------
elsif ReadData  = 4 then if WaitReady = STOP then ReadData <= 5; 
                                                  LCDRW <= '1';
                                                  LCD_A <= CD;
                                                  LCD_CE(0)<= L;
                                                  LCD_CE (1)<= R;
                                                  LCD_E <= '1';
                                             end if;                   
                         ----------------------------------------------
elsif ReadData  = 5 then LCD_E <= '0';
                         ReadData <= STOP;
                     end if;
--=====================================================================
   if WaitReady = 1 then LCDRW <= '1';
                         LCD_A <='0';
                         LCD_CE(0) <= L;
                         LCD_CE(1) <= R;
                         WaitReady <= 2;
                         ----------------------------------------------
elsif WaitReady = 2 then LCD_E <= '1';
                         WaitReady <= 3; 
                         ----------------------------------------------
elsif WaitReady = 3 then if DataR(7) = '0' then WaitReady <= STOP;
                                                LCD_E <= '0';
                                           end if;              
                    end if;
--=====================================================================
end if;
end process;
------------



LCD_RW <= LCDRW;

--=====================================================================
LCD_D <= DataW when LCDRW = '0' else (others => 'Z');
DataR <= LCD_D;
--=====================================================================

end architecture G;


