------------------------------------------------+
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use std.standard.all;
------------------------------------------------+
entity  MenuLCD is

Port (  STATUS:         in   std_logic;
		Clk:            in   std_logic;
        KEY:            in   unsigned (4 downto 0);
        RamDataRead:    in   unsigned (7 downto 0);
        RomData:        in   unsigned (7 downto 0);
		----------------------------------------
		X:              out  unsigned(7 downto 0) :=x"00";
		Y:              out  unsigned(7 downto 0) :=x"00";
		Point_DATA:     out  std_logic :='0';
		Point_START:    out  std_logic :='0';
		Byte_START:     out  std_logic :='0';
        RamDataWrite:   out  unsigned (7 downto 0);
        RamAddress:     out  integer range 0 to 1023;
        RamRW:          out  std_logic :='0';
        RomAddress:     out  integer range 0 to 1023;
		LED:            out  unsigned(7 downto 0) :=x"00";
		LCD_LED:        out  std_logic :='1';
		BUZZER:         out  std_logic :='0'
        );

end MenuLCD;
------------------------------------------------+
architecture G of MenuLCD is
--=============================================================
--        , :
--  type StatePosition is (STOP,START,.......);
--	signal StateXXXXX:         StatePosition := STOP;
--     START, STOP, 2,3,4,5.....
---------------------------------------------------
	constant STOP:                      integer := 0;
	constant START:                     integer := 1;
---------------------------------------------------
    signal KEYtmp, KEYo:                unsigned (4 downto 0);
    signal LEDtmp:                      unsigned (7 downto 0);
    signal LCD_LEDtmp:                  std_logic :='1';
    signal BUZZERtmp:                   std_logic :='0';
	signal KEY_LR:                      integer range 0 to 7   :=STOP;
	signal KEY_UD:                      integer range 0 to 7   :=STOP;
---------------------------------------------------
	signal PointX, PointY:              integer range 0 to 127 :=STOP;
	signal RAMaddr, RAMdata:            integer range 0 to 1023:=STOP;
    signal PointData:                   std_logic :='0';
---------------------------------------------------
	signal StateMenu, TextOut:          integer range 0 to 31  :=STOP;
	signal TextLenth:                   integer range 0 to 31  :=STOP;
	signal TxtExtPage:                  integer range 0 to 31  :=STOP;
	signal NumPage, NumberPage:         integer range 0 to 31  :=STOP;
	signal LevelPage:                   integer range 0 to 31  :=STOP;
	signal DeltaTextForLevel:           integer range 0 to 31  :=STOP;
	signal Stroka, Stolbec:             integer range 0 to 31  :=STOP;
	signal CursorX, CursorY:            integer range 0 to 127 :=STOP;
	signal CursorLenth, CursorWidth:    integer range 0 to 127 :=STOP;
	signal CursX, CursY, CursW, CursL:  integer range 0 to 127 :=STOP;
	signal CursPOS:                     integer range 0 to 7   :=STOP;
---------------------------------------------------
	signal TextPosX, TextPosY:          integer range 0 to 127:= 0;
	signal SymbolPos, BytePos, BitPos:  integer range 0 to 127:= 0;
--===      ========================
--===     ,   - ====
--===   ==============================================
	type TextLCD is array (0 to 31) of  string (1 to 10);    
	signal TXTrom:  TextLCD:= ( "<<  >>",-- 0
                                "",-- 1
                                "..",-- 2
                                " .",-- 3
                                "   UART   ",-- 4
                                -----------------
                                "==LOGIC===",-- 6
                                "27.10.2016",-- 5
                                "====",-- 7
                                "===1.00b==",-- 8
                               --0123456789------
                                " ETHERNET ",-- 9
                                "  UART 0  ",-- 10
                                " KEYBOARD ",-- 11
                                "  MOUSE   ",-- 12
                               ------------------
                                "   I2C    ",-- 13
                                "   SPI    ",-- 14
                                "   ADC    ",-- 15
                                "   DAC    ",-- 16
                               --0123456789------
                                "   LDM-SYS",-- 17
                                "TEMS.RU   ",-- 18
                               --0123456789------
                                "<< UART >>",-- 19
                                "  ",-- 20
                                "-  ",-- 21
                                "- ",-- 22
                                "     ",-- 23
                                -----------------
                                "  921600  ",-- 24
                                "    1     ",-- 25
                                "    8     ",-- 26
                                "          ",-- 27
                               --0123456789------
                      others => "HELLO TREE");
    signal TXT:                         unsigned (79 downto 0);
	signal TXT_ROM:                     string (1 to 10);
    signal TxtRomAddress:               integer range 0 to 31;
--=============================================================
BEGIN



-----------------
process (Clk)
variable RomAddressTMP:					integer range 0 to 2047;
variable RomAddressTEMP:				integer range 0 to 2047;
variable TXTtmp:				        unsigned (79 downto 0);
begin
if Clk'event and Clk = '1' then
--=====================================================================


-- KEY(1) UP
-- KEY(2) DOWN
-- KEY(0) SELECT
-- KEY(4) RIGHT
-- KEY(3) LEFT




--=====================================================================
NumberPage <= 0; --    
KEYtmp <= KEY;
KEYo <= KEYtmp and not (KEY);
---------------------------
   if KEYo(1) = '1' then KEY_UD <= KEY_UD - 1;
elsif KEYo(2) = '1' then KEY_UD <= KEY_UD + 1;
                    end if;
---------------------------
   if KEYo(4) = '1' then KEY_LR <= KEY_LR + 1;
elsif KEYo(3) = '1' then KEY_LR <= KEY_LR - 1;
                    end if;
---------------------------
CursPOS <= KEY_UD + (KEY_LR * 4);
--- ------------------------
   if CursPOS = 0 then CursorLenth <= 64; CursorWidth <= 10; CursorX <= 0;  CursorY <= 10;
elsif CursPOS = 1 then CursorLenth <= 64; CursorWidth <= 10; CursorX <= 0;  CursorY <= 21;
elsif CursPOS = 2 then CursorLenth <= 64; CursorWidth <= 10; CursorX <= 0;  CursorY <= 32;
elsif CursPOS = 3 then CursorLenth <= 64; CursorWidth <= 10; CursorX <= 0;  CursorY <= 43;
elsif CursPOS = 4 then CursorLenth <= 64; CursorWidth <= 10; CursorX <= 64; CursorY <= 10;
elsif CursPOS = 5 then CursorLenth <= 64; CursorWidth <= 10; CursorX <= 64; CursorY <= 21;
elsif CursPOS = 6 then CursorLenth <= 64; CursorWidth <= 10; CursorX <= 64; CursorY <= 32;
elsif CursPOS = 7 then CursorLenth <= 64; CursorWidth <= 10; CursorX <= 64; CursorY <= 43;
                  end if;
--=====================================================================


LED <= LEDtmp;
LCD_LED <= LCD_LEDtmp;
BUZZER <= BUZZERtmp;

--   -----------------
   if LevelPage = 0 
 then if CursPOS = 0 and KEYo(0) = '1' then LEDtmp <= LEDtmp + x"01";
   elsif CursPOS = 1 and KEYo(0) = '1' then LCD_LEDtmp <= not LCD_LEDtmp;
   elsif CursPOS = 2 and KEYo(0) = '1' then BUZZERtmp  <= not BUZZERtmp;
   elsif CursPOS = 3 and KEYo(0) = '1' then LevelPage <= 1; 
                                       end if;
 else if CursPOS = 3 and KEYo(0) = '1' then LevelPage <= 0; 
                                       end if;
 end if;
------------------------------------------   






--=====================================================================
case StateMenu is
              ---   --------------------------
    when STOP => StateMenu <= START;
                 Point_START <= '0';
                 Byte_START  <= '0';
                 RamRW <= '0';
                 RAMdata <= 0;
                 RAMaddr <= 0;
              ---  -------------------------------
    when START => StateMenu <= 2;
                  RamRW <= '0';
                  if PointX >= 127 
                then PointX <= 0;
                     if PointY >= 63 
                   then PointY <= 0;
                        StateMenu <= 3;
                   else PointY <= PointY + 1;
                        StateMenu <= 2;
                   end if;
                else PointX <= PointX + 1;
                     StateMenu <= 2;
                end if;
              --- ,  ---------------------------
    when 2 => RamDataWrite (PointX) <= '0';
              RamAddress <= (PointY * 16) + (PointX / 8);
              RamRW <= '1';
              StateMenu <= START;
              ---     ---------------------
    when 3 => if LevelPage = 0 then DeltaTextForLevel <= 0;
                               else DeltaTextForLevel <= 19;
                               end if;
              TxtExtPage <= START;
              NumPage <= NumberPage;
              StateMenu <= StateMenu + 1;
              --- ,  --------------------------
    when 4 => if TxtExtPage = STOP then StateMenu <= StateMenu + 1;
                                   end if;
              ---     ----------------
    when 5 =>  CursX <= CursorX;
               CursL <= CursorLenth;
               CursY <= CursorY;
               CursW <= CursorWidth;
               RAMaddr <= 0;
               RAMdata <= 0;
               PointY <= 0;
               PointX <= 0;
               StateMenu <= StateMenu + 1;
              ---     -----------------
    when 6 =>  Point_START <= '0';
               if PointX >= 127 
             then PointX <= 0;
                  if PointY >= 63 
                then PointY <= 0;
                     StateMenu <= STOP;
                else PointY <= PointY + 1;
                     StateMenu <= 7;
                end if;
             else PointX <= PointX + 1;
                  StateMenu <= 7;
             end if;
              ---     --
    when 7 =>  if STATUS = '1' 
             then Point_START <= '1';
                  if PointY < 9  
                then PointData <= not RamDataRead(PointX);
                ------------------------------------------
               elsif PointY > 54 
                then PointData <= not RamDataRead(PointX);
                ------------------------------------------
               elsif PointX >= CursX + 0 
                 and PointX <= CursX + (CursL - 2)
                 and PointY >= CursY + 1 
                 and PointY <= CursY + (CursW - 1)
                then PointData <= not RamDataRead(PointX);
                ------------------------------------------
               elsif PointX >= CursX + 1 
                 and PointX <= CursX + (CursL - 3) 
                 and PointY = CursY
                then PointData <= not RamDataRead(PointX);
                ------------------------------------------
               elsif PointX >= CursX + 1 
                 and PointX <= CursX + (CursL - 3) 
                 and PointY = CursY + CursW
                then PointData <= not RamDataRead(PointX);
                ------------------------------------------
                else PointData <= RamDataRead(PointX);
                end if;
                  RamAddress <= (PointY*16)+(PointX/8);
                  RamRW <= '0';
                  StateMenu <= 8;
             end if;
              ---------------------------------------------------------
    when 8 =>  if STATUS = '0' then StateMenu <= 6; 
                                    Point_START <= '0'; 
                               end if;
              ---------------------------------------------------------
    when others => StateMenu <= STOP;
end case;
--=====================================================================













--=== ======================================================
case TxtExtPage is
    when STOP => TextLenth <= 10;
              ---------------------------------------------------------
    when START => if Stolbec <= 1 
                then if Stroka = 0 
                   then if TextOut = STOP 
                      then TextPosX  <= 31; 
                           TextPosY  <= 1;
                           TxtRomAddress <= DeltaTextForLevel;
                           TxtExtPage <= 2;
                      end if;
                  elsif Stroka <=4 
                   then if TextOut = STOP 
                      then TextPosX  <= Stolbec * 64; 
                           TextPosY  <= TextPosY + 11;
                           TxtRomAddress <=(NumPage * 8) + Stroka + DeltaTextForLevel + (Stolbec * 4);
                           TxtExtPage <= 2;
                      end if;
                   else Stroka <= 0;
                        Stolbec <= Stolbec + 1;
                   end if;
                else Stolbec <= 0;
                     TxtExtPage <= 4;
                end if;
              ---------------------------------------------------------
    when 2 => TxtExtPage <= 3;
              ---------------------------------------------------------
    when 3 => TextOut <= START;
              Stroka <= Stroka + 1;
              TXT <= to_unsigned(string'pos(TXT_ROM),80);
              TxtExtPage <= START;
              ---------------------------------------------------------
    when 4 => if TextOut = STOP 
            then TxtExtPage <= 5;
            end if;
              ---------------------------------------------------------
    when 5 => TextOut <= START;
               TextPosX  <= 5;
               TextPosY  <= 56;
               TxtRomAddress <= 17;
               TxtExtPage <= 6;
              ---------------------------------------------------------
    when 6 => TXT <= to_unsigned(string'pos(TXT_ROM),80);
               if TextOut = STOP
             then TextOut <= START;
                  TextPosX  <= 65;
                  TextPosY  <= 56;
                  TxtRomAddress <= 18;
                  TxtExtPage <= 7;
             end if;
              ---------------------------------------------------------
    when 7 => TXT <= to_unsigned(string'pos(TXT_ROM),80);
               if TextOut = STOP
             then TxtExtPage <= STOP;
             end if;
              ---------------------------------------------------------
    when others => TxtExtPage <= STOP;
end case;
--=====================================================================



















--=== ======================================================
case TextOut is
    when STOP => SymbolPos <= STOP;
                 BytePos <= STOP;
                 BitPos <= STOP;
              ---------------------------------------------------------
    when START => RamRW <= '0';
                  if SymbolPos >= TextLenth 
                then TextOut <= STOP;
                     SymbolPos <= STOP;
                else if BytePos >= 5 
                   then BytePos <= STOP;
                        SymbolPos <= SymbolPos + 1;
                   else if BitPos >= 8 
                      then BitPos <= STOP;
                           BytePos <= BytePos + 1;
                      else TXTtmp := shift_right(TXT,SymbolPos * 8);
                           RomAddressTMP := to_integer(TXTtmp (7 downto 0));
                           if RomAddressTMP > 127 then RomAddressTEMP := RomAddressTMP - 63;
												  else RomAddressTEMP := RomAddressTMP;
												  end if;
						   RomAddress <= (5 * RomAddressTEMP) + BytePos;
                           RAMdata <= TextPosX + ((TextLenth - SymbolPos) * 6) + BytePos - 4;
                           TextOut <= 2;
                      end if;
                   end if;
                end if;
              ---------------------------------------------------------
    when 2  =>    if RAMdata >= 0   and RAMdata <= 8   then RAMaddr <= 0;
               elsif RAMdata >= 9   and RAMdata <= 16  then RAMaddr <= 1;
               elsif RAMdata >= 17  and RAMdata <= 24  then RAMaddr <= 2;
               elsif RAMdata >= 25  and RAMdata <= 32  then RAMaddr <= 3;
               elsif RAMdata >= 33  and RAMdata <= 40  then RAMaddr <= 4;
               elsif RAMdata >= 41  and RAMdata <= 48  then RAMaddr <= 5;
               elsif RAMdata >= 49  and RAMdata <= 56  then RAMaddr <= 6;
               elsif RAMdata >= 57  and RAMdata <= 64  then RAMaddr <= 7;
               elsif RAMdata >= 65  and RAMdata <= 72  then RAMaddr <= 8;
               elsif RAMdata >= 73  and RAMdata <= 80  then RAMaddr <= 9;
               elsif RAMdata >= 81  and RAMdata <= 88  then RAMaddr <= 10;
               elsif RAMdata >= 89  and RAMdata <= 96  then RAMaddr <= 11;
               elsif RAMdata >= 97  and RAMdata <= 104 then RAMaddr <= 12;
               elsif RAMdata >= 105 and RAMdata <= 112 then RAMaddr <= 13;
               elsif RAMdata >= 113 and RAMdata <= 120 then RAMaddr <= 14;
               elsif RAMdata >= 121 and RAMdata <= 127 then RAMaddr <= 15;
                                                       end if;
               TextOut <= 3;
              ---------------------------------------------------------
    when 3  => RamAddress <= ((TextPosY + BitPos) * 16) + RAMaddr;
               TextOut <= 4; 
              ---------------------------------------------------------
    when 4  => TextOut <= 5;
              ---------------------------------------------------------
    when 5  => RamDataWrite <= RamDataRead;
               TextOut <= 6;
              ---------------------------------------------------------
    when 6  => RamDataWrite(RAMdata) <= RomData(BitPos);
               TextOut <= 7;
              ---------------------------------------------------------
    when 7  => RamRW <= '1';
               BitPos <= BitPos + 1;
               TextOut <= START;
              ---------------------------------------------------------
    when others => TextOut <= STOP;
end case;
--=====================================================================




--=====================================================================
TXT_ROM <= TXTrom(TxtRomAddress);
--=====================================================================





--=====================================================================
end if;
end process;
------------------










--=====================================================================
X <= to_unsigned (PointX,8);
Y <= to_unsigned (PointY,8);
Point_DATA <= PointData;
--=====================================================================




end architecture G;