---------------------------------------------------------------------------+
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
---------------------------------------------------------------------------+
entity  RS232 is

generic(BAUD_BitInSec:  natural range 1 to 921600 := 921600;
        CLK_InHz:       natural range 1 to 100000000 := 50000000);
        
port   (RXD:                in   std_logic :='1';
		WriteDATA:          in   unsigned(7 downto 0);
		WriteSTART:         in   std_logic :='0';
		CLK:                in   std_logic;
		TXD:                out  std_logic :='1';
		ReadDATA:           out  unsigned(7 downto 0);
		ReadSTART:          out  std_logic :='0';
		ReadSTOP:           out  std_logic :='0';
		WriteSTOP:          out  std_logic :='1');
end RS232;
---------------------------------------------------------------------------+
architecture G of RS232 is
--=========================================================================
    type StatePosition is (STOP,START);
	signal StateRX,StateTX:         StatePosition := STOP;
	-----------------------------------------------------------------------
	constant SampleOnBit:           integer range 0 to 15 := 7;
	signal C:                       integer range 0 to CLK_InHz-1 :=0;
	signal Rtakt,Rbit:              integer range 0 to 15 :=0;
	signal Ttakt,Tbit:              integer range 0 to 15 :=0;
	signal TxData,RxData:           unsigned(7 downto 0);
	signal R:                       unsigned(7 downto 0);
	signal CLK_UART:                std_logic :='0';
--=========================================================================
BEGIN




---------------------------------------------------------------------------+
process(CLK)
--=========================================================================
variable T: integer;
--=========================================================================
begin
if CLK'Event and CLK='1' then
---Sintezator50MHz----------------+
T:=C + (BAUD_BitInSec * SampleOnBit);--one step = 1Hz
if T >= (CLK_InHz-1) then C <=(T-(CLK_InHz-1));
                     else C <= T;
                     end if;
----------------------------------+
if C>=0 and C<=((CLK_InHz/2)-1) then CLK_UART <= '1';
                                else CLK_UART <= '0';
                                end if;
----------------------------------+
end if;
end process;
---------------------------------------------------------------------------+





---------------------------------------------------------------------------+
process (CLK_UART)
begin
if CLK_UART'event and CLK_UART = '1' then
----------------------------------------------+
case StateRX is
    ------------------------------------------
    when STOP   => if RXD = '0' then if Rtakt >= (SampleOnBit - 2) then Rtakt <= 0;
                                                                        Rbit <= 0;
                                                                        StateRx <= START;
                                                                   else Rtakt <= Rtakt + 1;
                                                                   end if;
                                     ReadSTOP <= '0';
                                else ReadSTOP <= '1';
                                end if;
    ------------------------------------------
    when START  => if Rbit >= 8 then if Rtakt >= (SampleOnBit - 1)         then Rtakt <= 0;
                                                                                Rbit <= 0;
                                                                                StateRx <= STOP;
                                                                                ReadSTART <= '0';
                                  elsif Rtakt  = (((SampleOnBit - 1)/2)+1) then Rtakt <= Rtakt + 1;
                                                                                ReadSTART <= '1';
                                  elsif Rtakt  = ((SampleOnBit - 1)/2)     then if RXD = '1' 
                                                                              then Rtakt <= Rtakt + 1;
                                                                                   ReadDATA <= RxData;
                                                                              else StateRx <= STOP;
                                                                              end if;
                                                                           else Rtakt <= Rtakt + 1;
                                                                           end if;
                                else if Rtakt >= (SampleOnBit - 1)         then Rtakt <= 0;
                                                                                Rbit <= Rbit + 1;
                                  elsif Rtakt  = ((SampleOnBit - 1)/2)     then Rtakt <= Rtakt + 1;
                                                                                RxData(Rbit) <= RXD; 
                                                                           else Rtakt <= Rtakt + 1;
                                                                           end if; 
                                end if; 
    ------------------------------------------
    when others => StateRX <= STOP;
end case;
----------------------------------------------+
case StateTX is
    ------------------------------------------
    when STOP   => if WriteSTART = '1' then Ttakt <= 0;
                                            Tbit <= 0;
                                            StateTx <= START;
                                            WriteSTOP <= '0';
                                            TxData <= WriteDATA;
                                       end if;
    ------------------------------------------
    when START =>  if Tbit >= 9 then if Ttakt >= (SampleOnBit - 3) then Ttakt <= 0;
                                                                        Tbit <= 0;
                                                                        WriteSTOP <= '1';
                                                                        StateTx <= STOP;
                                                                        TXD <= '1';
                                                                   else Ttakt <= Ttakt + 1;
                                                                        TXD <= '1';
                                                                   end if;
                elsif Tbit  = 0 then if Ttakt >= (SampleOnBit - 1) then Ttakt <= 0;
                                                                        Tbit <= Tbit + 1;
                                                                   else Ttakt <= Ttakt + 1;
                                                                        TXD <= '0';
                                                                   end if;
                                else if Ttakt >= (SampleOnBit - 1) then Ttakt <= 0;
                                                                        Tbit <= Tbit + 1;
                                                                   else Ttakt <= Ttakt + 1;
                                                                        TXD <= TxData(Tbit - 1);
                                                                   end if; 
                                end if; 
    ------------------------------------------
    when others => StateTX <= STOP;
end case;
----------------------------------------------+
end if;
end process;
-------------------------------------------------------------------------------------------------+








end architecture G;